package pay.platform.mapper;

import com.mybatisflex.core.BaseMapper;
import com.mybatisflex.core.query.QueryWrapper;
import pay.platform.domain.SysPermission;
import pay.platform.domain.table.SysPermissionTableDef;
import pay.platform.domain.table.SysRolePermissionTableDef;
import jakarta.annotation.Nonnull;
import jakarta.annotation.Nullable;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;
import java.util.Optional;

/**
 * 系统 - 菜单与权限 映射层。
 *
 * @author admin
 * @since 1.0
 */
@Mapper
public interface SysPermissionMapper extends BaseMapper<SysPermission> {

    /**
     * 根据角色ID查询菜单与权限
     *
     * @param isEnabled 是否查询启用的
     * @return 菜单与权限
     */
    default List<SysPermission> listByRoleIds(@Nullable Boolean isEnabled) {
        Integer enable = Optional.ofNullable(isEnabled).map(e -> e ? 1 : 0).orElse(0);
        QueryWrapper query = new QueryWrapper()
                .select()
                .from(SysPermissionTableDef.SYS_PERMISSION, SysRolePermissionTableDef.SYS_ROLE_PERMISSION)
                .where(SysPermissionTableDef.SYS_PERMISSION.ID.eq(SysRolePermissionTableDef.SYS_ROLE_PERMISSION.PERMISSION_ID))
                .and(SysPermissionTableDef.SYS_PERMISSION.ENABLE.eq(enable));
        return selectListByQuery(query);
    }

    /**
     * 根据角色ID查询菜单与权限
     *
     * @param roleIds   角色ID
     * @param isEnabled 是否查询启用的
     * @return 菜单与权限
     */
    default List<SysPermission> listByRoleIds(@Nonnull List<String> roleIds, @Nullable Boolean isEnabled) {
        Integer enable = Optional.ofNullable(isEnabled).map(e -> e ? 1 : 0).orElse(0);
        QueryWrapper query = new QueryWrapper()
                .select()
                .from(SysPermissionTableDef.SYS_PERMISSION.as("p"), SysRolePermissionTableDef.SYS_ROLE_PERMISSION.as("t"))
                .where(SysPermissionTableDef.SYS_PERMISSION.ID.eq(SysRolePermissionTableDef.SYS_ROLE_PERMISSION.PERMISSION_ID))
                .and(SysRolePermissionTableDef.SYS_ROLE_PERMISSION.ROLE_ID.in(roleIds))
                .and(SysPermissionTableDef.SYS_PERMISSION.ENABLE.eq(enable));
        return selectListByQuery(query);
    }

    /**
     * 菜单与权限
     *
     * @param isEnabled 是否查询启用的
     * @return 菜单与权限
     */
    default List<SysPermission> list(@Nullable Boolean isEnabled) {
        Integer enable = Optional.ofNullable(isEnabled).map(e -> e ? 1 : 0).orElse(0);
        QueryWrapper query = new QueryWrapper()
                .select()
                .from(SysPermissionTableDef.SYS_PERMISSION)
                .where(SysPermissionTableDef.SYS_PERMISSION.ENABLE.eq(enable));
        return selectListByQuery(query);
    }

    /**
     * 查询权限
     *
     * @param ids       ID集合
     * @param isEnabled 是否查询已启用的
     * @return 权限信息
     */
    default List<SysPermission> listPermission(@Nullable List<String> ids, @Nullable Boolean isEnabled) {
        Integer enable = Optional.ofNullable(isEnabled).map(e -> e ? 1 : 0).orElse(0);
        QueryWrapper query = new QueryWrapper()
                .select()
                .from(SysPermissionTableDef.SYS_PERMISSION)
                .where(SysPermissionTableDef.SYS_PERMISSION.ID.in(ids))
                .and(SysPermissionTableDef.SYS_PERMISSION.MENU_TYPE.eq(2))
                .and(SysPermissionTableDef.SYS_PERMISSION.ENABLE.eq(enable));
        return selectListByQuery(query);
    }

    /**
     * 查询菜单根据ID
     *
     * @param ids       菜单ID集合
     * @param isEnabled 是否查询启用
     * @return 菜单集合
     */
    @Nullable
    default List<SysPermission> listMenuByIds(@Nullable List<String> ids, @Nullable Boolean isEnabled) {
        Integer enable = Optional.ofNullable(isEnabled).map(e -> e ? 1 : 0).orElse(0);
        QueryWrapper query = new QueryWrapper()
                .select()
                .from(SysPermissionTableDef.SYS_PERMISSION)
                .where(SysPermissionTableDef.SYS_PERMISSION.MENU_TYPE.in(List.of(0, 1)))
                .and(SysPermissionTableDef.SYS_PERMISSION.ENABLE.eq(enable))
                .and(SysPermissionTableDef.SYS_PERMISSION.ID.in(ids))
                .orderBy(SysPermissionTableDef.SYS_PERMISSION.RANK.asc());
        return selectListByQuery(query);
    }
}
