package pay.platform.api.system.controller;

import cn.hutool.core.collection.CollectionUtil;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import pay.platform.api.system.model.query.RoleQuery;
import pay.platform.api.system.servcie.SysRolePermissionService;
import pay.platform.api.system.servcie.SysRoleService;
import pay.platform.core.common.Result;
import pay.platform.core.common.domain.BasePage;
import pay.platform.core.security.util.SecurityUtil;
import pay.platform.domain.SysRole;
import pay.platform.domain.SysRolePermission;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 系统角色
 */
@RestController
@RequestMapping("/sys/role")
@Tag(name = "系统：系统角色")
@RequiredArgsConstructor
public class SysRoleController {

    private final SysRoleService sysRoleService;

    private final SysRolePermissionService sysRolePermissionService;

    @GetMapping("/query/page")
    @PreAuthorize("@permission.hashPermission('role:query')")
    @Operation(summary = "分页查询")
    public Result<BasePage<SysRole>> queryPage(@ParameterObject RoleQuery query) {
        BasePage<SysRole> page = sysRoleService.queryPage(query);
        return Result.OK(page);
    }

    @GetMapping("/query/list")
    @Operation(summary = "列表查询")
    public Result<List<SysRole>> queryList(@ParameterObject RoleQuery query) {
        List<SysRole> roles = sysRoleService.queryList(query);
        return Result.OK(roles);
    }

    @GetMapping("/check/code")
    @Operation(summary = "校验code是否重复", parameters = {
            @Parameter(name = "id", required = false, description = "id"),
            @Parameter(name = "code", required = true, description = "角色编码")
    })
    public Result<Boolean> checkRole(@RequestParam(value = "id", required = false) String id,
                                     @RequestParam(value = "code") String code) {
        boolean checkCode = sysRoleService.checkCode(code, id);
        return Result.OK(checkCode);
    }

    @PostMapping("/save")
    @PreAuthorize("@permission.hashPermission('role:save')")
    @Operation(summary = "新增")
    public Result<SysRole> save(@Valid @RequestBody SysRole role) {
        String username = SecurityUtil.getCurrentUsername();
        role.setCreateUser(username);
        role.setUpdateUser(username);
        return sysRoleService.saveRole(role);
    }

    @PutMapping("/update")
    @PreAuthorize("@permission.hashPermission('role:update')")
    @Operation(summary = "修改")
    public Result<SysRole> update(@RequestParam("id") String id, @Valid @RequestBody SysRole role) {
        String username = SecurityUtil.getCurrentUsername();
        role.setId(id);
        role.setUpdateUser(username);
        return sysRoleService.updateRole(role);
    }

    @GetMapping("/permission")
    @Operation(summary = "获取权限ID", parameters = {
            @Parameter(required = true, description = "角色ID", name = "id")
    })
    public Result<List<String>> getPermissionIdsByRoleId(@RequestParam("id") String id) {
        Optional<List<String>> optional = sysRolePermissionService.findPermissionIdByRole(id);
        return Result.OK(optional.orElse(Collections.emptyList()));
    }

    @PutMapping("/permission")
    @Operation(summary = "更新角色权限")
    @PreAuthorize("@permission.hashPermission('role:permission')")
    public Result<Boolean> updatePermissionByRoleId(@Parameter(description = "角色ID") @RequestParam("id") String id,
                                                    @RequestBody List<String> permissionIds) {
        String username = SecurityUtil.getCurrentUsername();
        List<SysRolePermission> permissions = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(permissionIds)) {
            permissions = permissionIds.stream().map(permissionId -> {
                SysRolePermission rolePermission = new SysRolePermission();
                rolePermission.setPermissionId(permissionId);
                rolePermission.setRoleId(id);
                rolePermission.setUpdateUser(username);
                rolePermission.setUpdateUser(username);
                return rolePermission;
            }).collect(Collectors.toList());
        }
        return Result.OK(sysRolePermissionService.updatePermissionByRoles(id, permissions));
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除角色")
    @PreAuthorize("@permission.hashPermission('role:del')")
    public Result<String> delRole(HttpServletRequest request, @Valid @RequestBody List<String> ids) {
        return sysRoleService.deleteByIds(ids);
    }
}
