package pay.platform.api.system.servcie;


import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.query.QueryWrapper;
import pay.platform.api.system.event.RefreshOnlineUserRouteEvent;
import pay.platform.core.common.domain.BaseService;
import pay.platform.domain.SysRolePermission;
import pay.platform.domain.table.SysRolePermissionTableDef;
import pay.platform.mapper.SysRolePermissionMapper;
import jakarta.annotation.Nonnull;
import jakarta.annotation.Nullable;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 系统 - 角色权限 服务层实现。
 *
 * @author ADMIN
 * @since 1.0
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class SysRolePermissionService extends BaseService<SysRolePermissionMapper, SysRolePermission> {

    private final ApplicationContext applicationContext;

    /**
     * 获取角色对应的权限
     *
     * @param id 角色ID
     * @return 权限ID
     */
    public Optional<List<String>> findPermissionIdByRole(String id) {
        return Optional.ofNullable(mapper.getPermissionIdByRoleId(id));
    }

    /**
     * 更新角色权
     *
     * @param id            id
     * @param permissionIds 权限集合，如果权限为空，则该角色没有权限,<b>只有最后一级</b>
     * @return .
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updatePermissionByRoles(@Nonnull String id, @Nullable List<SysRolePermission> permissionIds) {
        if (CollectionUtil.isEmpty(permissionIds)) {
            // 删除所有权限
            QueryWrapper queryWrapper = QueryWrapper.create().from(SysRolePermissionTableDef.SYS_ROLE_PERMISSION).where(SysRolePermissionTableDef.SYS_ROLE_PERMISSION.ROLE_ID.eq(id));
            return mapper.deleteByQuery(queryWrapper) > 0;
        }
        // 根据最后一级查找所有的父级
        List<String> permissionIdList = permissionIds.stream().map(SysRolePermission::getPermissionId).toList();
        Set<String> parentContainer = CollectionUtil.newHashSet();
        parentContainer = getParentId(permissionIdList, parentContainer);
        parentContainer.addAll(permissionIdList);
        parentContainer = parentContainer.stream().filter(ObjectUtil::isNotEmpty).collect(Collectors.toSet());
        // 删除所有权限
        QueryWrapper queryWrapper = QueryWrapper.create().from(SysRolePermissionTableDef.SYS_ROLE_PERMISSION).where(SysRolePermissionTableDef.SYS_ROLE_PERMISSION.ROLE_ID.eq(id));
        mapper.deleteByQuery(queryWrapper);
        // 保存权限
        List<SysRolePermission> rolePermissions = parentContainer.stream().map(permissionId -> {
            SysRolePermission rolePermission = new SysRolePermission();
            rolePermission.setRoleId(id);
            rolePermission.setPermissionId(permissionId);
            rolePermission.setCreateTime(LocalDateTime.now());
            rolePermission.setCreateUser(permissionIds.getFirst().getCreateUser());
            return rolePermission;
        }).toList();
        boolean success = saveBatch(rolePermissions);
        // 刷新在线用户权限
        applicationContext.publishEvent(new RefreshOnlineUserRouteEvent(this));
        return success;
    }

    private Set<String> getParentId(List<String> permissionIdList, Set<String> container) {
        List<String> parentIds = mapper.getParentIdByPermissionIds(permissionIdList);
        if (CollectionUtil.isNotEmpty(parentIds)) {
            container.addAll(parentIds);
            return getParentId(parentIds, container);
        }
        return container;
    }
}