package pay.platform.api.servcie.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.RandomUtil;
import com.alipay.api.AlipayConstants;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pay.platform.api.model.request.PayRequest;
import pay.platform.api.model.response.PayResponse;
import pay.platform.api.servcie.*;
import pay.platform.core.exception.BusinessException;
import pay.platform.core.utils.QRCodeUtil;
import pay.platform.core.utils.SignUtil;
import pay.platform.domain.*;
import pay.platform.payment.alipay.AlipayClient;
import pay.platform.payment.alipay.core.AlipayCore;
import pay.platform.payment.alipay.entity.request.AlipayQrcodeRequest;
import pay.platform.payment.alipay.entity.response.AlipayQrcodeResponse;

import java.util.List;
import java.util.TreeMap;

/**
 *
 **/
@Service
public class PayServiceImpl implements PayService {

    @Autowired
    private PayMerchantAppKeyService payMerchantAppKeyService;
    @Autowired
    private PayOrderService payOrderService;
    @Autowired
    private PayAggregationStoreService payAggregationStoreService;
    @Autowired
    private PayChannelService payChannelService;
    @Autowired
    private PayAgentService payAgentService;
    @Autowired
    private PayMerchantService payMerchantService;

    @Override
    public PayResponse pay(PayRequest payRequest) {

        PayMerchantAppKey payMerchantAppKey = validateMerchant(payRequest);
        //验证签名
        validSign(payRequest, payMerchantAppKey.getAppKey());
        String agentId = payMerchantAppKey.getAgentId();
        //通过代理获取支付通道-》聚合支付-》店铺
        PayChannel payChannel = randomChannel(agentId);
        //轮循店铺
        PayAggregationStore payAggregationStore = randomStore(payChannel.getId());
        //创建本地订单
        PayOrder payOrder = createOrder(payRequest, payMerchantAppKey);
        //创建支付订单
        AlipayQrcodeResponse alipayQrcodeResponse = createPayOrder(payChannel, payOrder, payAggregationStore);
        return PayResponse.builder().qrCode(alipayQrcodeResponse.getQrCode()).imageBase64(QRCodeUtil.qrCodeBase64(alipayQrcodeResponse.getQrCode())).build();
    }

    private void validSign(PayRequest payRequest, String appKey) {
        TreeMap<String, String> treeMap = new TreeMap<>();
        treeMap.put("appId", payRequest.getAppId());
        treeMap.put("body", payRequest.getBody());
        treeMap.put("subject", payRequest.getSubject());
        treeMap.put("notifyUrl", payRequest.getNotifyUrl());
        treeMap.put("amount", payRequest.getAmount().toString());
        treeMap.put("orderNo", payRequest.getOrderNo());
        treeMap.put("currentTime", payRequest.getCurrentTime().toString());
        // 校验商户
        String sign = SignUtil.getSign(appKey, treeMap);
        if (!StringUtils.equals(payRequest.getSign(), sign)) {
            throw new BusinessException("签名错误");
        }
    }

    public AlipayQrcodeResponse createPayOrder(PayChannel payChannel, PayOrder payOrder, PayAggregationStore payAggregationStore) {
        AlipayCore.ClientBuilder clientBuilder = new AlipayCore.ClientBuilder();
        //signType=rsa时，可不传 alipay_public_key
        //alipayCore = clientBuilder.setAppId(appid).setPrivateKey(private_key).setSignType(AlipayConstants.SIGN_TYPE_RSA).build();

        //signType=rsa2时，必须传 支付宝公钥 alipay_public_key
        AlipayCore alipayCore = clientBuilder
                .setAlipayPublicKey(payChannel.getPublicKey())
                .setAppId(payChannel.getAppId())
                .setPrivateKey(payChannel.getPrivateKey())
                .setSignType(AlipayConstants.SIGN_TYPE_RSA2)
                .setSandbox(true)
                .build();
        AlipayQrcodeRequest request = new AlipayQrcodeRequest();
        request.setSysServiceProviderId(null);
        request.setSubject(payOrder.getSubject());
        request.setOutTradeNo(payOrder.getOrderNo());
        request.setBody(payOrder.getRemark());
        request.setOperatorId(payAggregationStore.getOperatorId());
        request.setStoreId(payAggregationStore.getStoreId());
        request.setTotalAmount(payOrder.getAmount().toString());
        AlipayQrcodeResponse response = AlipayClient.preCreate(request, alipayCore);
        if (!response.isSuccess()) {
            //设置通道不可用
            payChannel.setStatus(0);
            payChannelService.updateById(payChannel);
            throw new RuntimeException(response.getSubMsg());
        }
        return response;
    }

    /**
     * 创建本地订单
     *
     * @param payRequest
     * @param payMerchantAppKey
     * @return
     */
    private PayOrder createOrder(PayRequest payRequest, PayMerchantAppKey payMerchantAppKey) {
        PayOrder payOrder = PayOrder.builder()
                .amount(payRequest.getAmount())
                .subject(payRequest.getSubject())
                .remark(payRequest.getBody())
                .agentId(payMerchantAppKey.getAgentId())
                .status(0)
                .orderNo(payRequest.getOrderNo())
                .merchantId(payMerchantAppKey.getMerchantId())
                .build();
        payOrderService.save(payOrder);
        return payOrder;
    }

    /**
     * 随机店铺
     *
     * @param channelId
     * @return
     */
    private PayAggregationStore randomStore(String channelId) {
        List<PayAggregationStore> aggregationStore = payAggregationStoreService.findAggregationStore(channelId);
        if (CollectionUtil.isEmpty(aggregationStore)) {
            throw new BusinessException("无可用店铺");
        }
        return aggregationStore.get(RandomUtil.randomInt(aggregationStore.size()));
    }

    /**
     * 随机通道
     *
     * @param agentId
     * @return
     */
    private PayChannel randomChannel(String agentId) {
        List<PayChannel> payChannels = payChannelService.findChannel(agentId);
        if (CollectionUtil.isEmpty(payChannels)) {
            throw new BusinessException("无可用通道");
        }
        return payChannels.get(RandomUtil.randomInt(payChannels.size()));
    }

    private PayMerchantAppKey validateMerchant(PayRequest payRequest) {
        PayMerchantAppKey payMerchantAppKey = payMerchantAppKeyService.getByAppId(payRequest.getAppId());
        if (payMerchantAppKey == null) {
            throw new BusinessException("appId不存在");
        }
        PayMerchant payMerchant = payMerchantService.getById(payMerchantAppKey.getMerchantId());
        if (payMerchant == null) {
            throw new BusinessException("商户不存在");
        }
        if (payMerchant.getStatus() == 0) {
            throw new BusinessException("商户已禁用");
        }
        String agentId = payMerchantAppKey.getAgentId();
        if (StringUtils.isBlank(agentId)) {
            throw new BusinessException("代理商不能为空");
        }
        PayAgent payAgent = payAgentService.getById(agentId);
        if (payAgent == null) {
            throw new BusinessException("代理不存在");
        }
        if (payAgent.getStatus() == 0) {
            throw new BusinessException("代理已禁用");
        }
        return payMerchantAppKey;
    }
}
