package pay.platform.core.utils;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.WriterException;
import com.google.zxing.common.BitMatrix;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

public class QRCodeUtil {
    private static final Logger logger = LoggerFactory.getLogger(QRCodeUtil.class);

    private static final int BLACK = 0xFF000000;
    private static final int WHITE = 0xFFFFFFFF;

    public static final String QRCODE_IMAGE_FORMAT = "png";


    /**
     * 保存二维码文件
     *
     * @param text     二维码内容
     * @param filePath 保存地址
     * @param width    图片宽度
     * @param height   图片高度
     * @return 保存结果
     */
    public static boolean saveQRCodeFile(String text, String filePath, int width, int height) {
        if (text == null || text.isEmpty()) {
            throw new IllegalArgumentException("QR code text can't be empty");
        }
        if (filePath == null || filePath.isEmpty()) {
            throw new IllegalArgumentException("QR code file path can't be empty");
        }

        Map<EncodeHintType, String> hints = new HashMap<>();
        hints.put(EncodeHintType.CHARACTER_SET, "UTF-8");

        BitMatrix bitMatrix;
        try {
            bitMatrix = new MultiFormatWriter().encode(text, BarcodeFormat.QR_CODE, width, height, hints);
        } catch (WriterException e) {
            logger.error("encode text with exception", e);
            return false;
        }

        File file = new File(filePath);
        if (file.exists()) {
            logger.error("QR code file is exist, path:{}", file.getAbsolutePath());
            return false;
        }

        try {
            return writeFile(bitMatrix, file);
        } catch (IOException e) {
            logger.error("save QR code file with exception, path:{}", file.getAbsolutePath(), e);
            return false;
        }
    }

    /**
     * 写二维码到输出流
     *
     * @param text   二维码内容
     * @param stream 输出流
     * @param width  图片宽度
     * @param height 图片高度
     * @return 处理结果
     */
    public static boolean writeQRCodeStream(String text, OutputStream stream, int width, int height) {
        if (text == null || text.isEmpty()) {
            throw new IllegalArgumentException("QR code text can't be empty");
        }
        if (stream == null) {
            throw new IllegalArgumentException("QR code OutputStream can't be null");
        }

        Map<EncodeHintType, String> hints = new HashMap<>();
        hints.put(EncodeHintType.CHARACTER_SET, "UTF-8");

        BitMatrix bitMatrix;
        try {
            bitMatrix = new MultiFormatWriter().encode(text, BarcodeFormat.QR_CODE, width, height, hints);
        } catch (WriterException e) {
            logger.error("encode text with exception", e);
            return false;
        }

        try {
            return writeStream(bitMatrix, stream);
        } catch (IOException e) {
            logger.error("write QR code stream with exception", e);
            return false;
        }
    }

    /* ...... */

    private static BufferedImage toBufferedImage(BitMatrix matrix) {
        int width = matrix.getWidth();
        int height = matrix.getHeight();
        BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        for (int x = 0; x < width; x++) {
            for (int y = 0; y < height; y++) {
                image.setRGB(x, y, matrix.get(x, y) ? BLACK : WHITE);
            }
        }
        return image;
    }

    private static boolean writeFile(BitMatrix matrix, File file) throws IOException {
        BufferedImage image = toBufferedImage(matrix);
        return ImageIO.write(image, QRCODE_IMAGE_FORMAT, file);
    }

    private static boolean writeStream(BitMatrix matrix, OutputStream stream) throws IOException {
        BufferedImage image = toBufferedImage(matrix);
        return ImageIO.write(image, QRCODE_IMAGE_FORMAT, stream);
    }

    /**
     * 写二维码到输出流
     *
     * @param text 二维码内容
     */
    public static String qrCodeBase64(String text) {
        int width = 300, height = 300;
        if (text == null || text.isEmpty()) {
            throw new IllegalArgumentException("QR code text can't be empty");
        }
        Map<EncodeHintType, String> hints = new HashMap<>();
        hints.put(EncodeHintType.CHARACTER_SET, "UTF-8");

        BitMatrix bitMatrix;
        try {
            bitMatrix = new MultiFormatWriter().encode(text, BarcodeFormat.QR_CODE, width, height, hints);
            BufferedImage bufferedImage = toBufferedImage(bitMatrix);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            ImageIO.write(bufferedImage, "png", baos); // 可以改为需要的图片格式，如"jpg"
            byte[] imageBytes = baos.toByteArray();
            return Base64.getEncoder().encodeToString(imageBytes);
        } catch (Exception e) {
            logger.error("encode text with exception", e);
            throw new RuntimeException(e);
        }

    }

    public static String toAscii(BitMatrix bitMatrix) {
        StringBuilder sb = new StringBuilder();
        for (int rows = 0; rows < bitMatrix.getHeight(); rows++) {
            for (int cols = 0; cols < bitMatrix.getWidth(); cols++) {
                boolean x = bitMatrix.get(rows, cols);
                if (!x) {
                    sb.append("\033[40m   \033[0m");
                } else {
                    sb.append("\033[38m   \033[0m");
                }
            }
            sb.append("\n");
        }
        return sb.toString();
    }
}
